# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['ZoneShareV2Args', 'ZoneShareV2']

@pulumi.input_type
class ZoneShareV2Args:
    def __init__(__self__, *,
                 target_project_id: pulumi.Input[_builtins.str],
                 zone_id: pulumi.Input[_builtins.str],
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a ZoneShareV2 resource.

        :param pulumi.Input[_builtins.str] target_project_id: The ID of the target project with which the
               DNS zone will be shared.
        :param pulumi.Input[_builtins.str] zone_id: The ID of the DNS zone to be shared.
        :param pulumi.Input[_builtins.str] project_id: The ID of the project DNS zone is created for, sets
               `X-Auth-Sudo-Tenant-ID` header (requires an assigned user role in target
               project).
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 DNS client. If
               omitted, the `region` argument of the provider is used. Changing this creates
               a new DNS zone share.
        """
        pulumi.set(__self__, "target_project_id", target_project_id)
        pulumi.set(__self__, "zone_id", zone_id)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter(name="targetProjectId")
    def target_project_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the target project with which the
        DNS zone will be shared.
        """
        return pulumi.get(self, "target_project_id")

    @target_project_id.setter
    def target_project_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "target_project_id", value)

    @_builtins.property
    @pulumi.getter(name="zoneId")
    def zone_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the DNS zone to be shared.
        """
        return pulumi.get(self, "zone_id")

    @zone_id.setter
    def zone_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "zone_id", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the project DNS zone is created for, sets
        `X-Auth-Sudo-Tenant-ID` header (requires an assigned user role in target
        project).
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 DNS client. If
        omitted, the `region` argument of the provider is used. Changing this creates
        a new DNS zone share.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class _ZoneShareV2State:
    def __init__(__self__, *,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 target_project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 zone_id: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering ZoneShareV2 resources.

        :param pulumi.Input[_builtins.str] project_id: The ID of the project DNS zone is created for, sets
               `X-Auth-Sudo-Tenant-ID` header (requires an assigned user role in target
               project).
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 DNS client. If
               omitted, the `region` argument of the provider is used. Changing this creates
               a new DNS zone share.
        :param pulumi.Input[_builtins.str] target_project_id: The ID of the target project with which the
               DNS zone will be shared.
        :param pulumi.Input[_builtins.str] zone_id: The ID of the DNS zone to be shared.
        """
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if target_project_id is not None:
            pulumi.set(__self__, "target_project_id", target_project_id)
        if zone_id is not None:
            pulumi.set(__self__, "zone_id", zone_id)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the project DNS zone is created for, sets
        `X-Auth-Sudo-Tenant-ID` header (requires an assigned user role in target
        project).
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 DNS client. If
        omitted, the `region` argument of the provider is used. Changing this creates
        a new DNS zone share.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="targetProjectId")
    def target_project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the target project with which the
        DNS zone will be shared.
        """
        return pulumi.get(self, "target_project_id")

    @target_project_id.setter
    def target_project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "target_project_id", value)

    @_builtins.property
    @pulumi.getter(name="zoneId")
    def zone_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the DNS zone to be shared.
        """
        return pulumi.get(self, "zone_id")

    @zone_id.setter
    def zone_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "zone_id", value)


@pulumi.type_token("openstack:dns/zoneShareV2:ZoneShareV2")
class ZoneShareV2(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 target_project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 zone_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Manages the sharing of a DNS zone in the OpenStack DNS Service.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_openstack as openstack

        example = openstack.dns.ZoneShareV2("example",
            zone_id="00000000-0000-0000-0000-000000000000",
            target_project_id="11111111-1111-1111-1111-111111111111",
            project_id="22222222-2222-2222-2222-222222222222")
        ```

        ## Import

        DNS zone share can be imported by specifying the zone ID with share ID and optional project ID:

        ```sh
        $ pulumi import openstack:dns/zoneShareV2:ZoneShareV2 share_1 60cbdc69-64f9-49ee-b294-352e71e22827/0e1dae51-aee2-4b44-962f-885bb69f3a5c
        $ pulumi import openstack:dns/zoneShareV2:ZoneShareV2 share_1 60cbdc69-64f9-49ee-b294-352e71e22827/0e1dae51-aee2-4b44-962f-885bb69f3a5c/eb92139f6c054a878852ac9e8cbe612a
        ```


        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] project_id: The ID of the project DNS zone is created for, sets
               `X-Auth-Sudo-Tenant-ID` header (requires an assigned user role in target
               project).
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 DNS client. If
               omitted, the `region` argument of the provider is used. Changing this creates
               a new DNS zone share.
        :param pulumi.Input[_builtins.str] target_project_id: The ID of the target project with which the
               DNS zone will be shared.
        :param pulumi.Input[_builtins.str] zone_id: The ID of the DNS zone to be shared.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ZoneShareV2Args,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages the sharing of a DNS zone in the OpenStack DNS Service.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_openstack as openstack

        example = openstack.dns.ZoneShareV2("example",
            zone_id="00000000-0000-0000-0000-000000000000",
            target_project_id="11111111-1111-1111-1111-111111111111",
            project_id="22222222-2222-2222-2222-222222222222")
        ```

        ## Import

        DNS zone share can be imported by specifying the zone ID with share ID and optional project ID:

        ```sh
        $ pulumi import openstack:dns/zoneShareV2:ZoneShareV2 share_1 60cbdc69-64f9-49ee-b294-352e71e22827/0e1dae51-aee2-4b44-962f-885bb69f3a5c
        $ pulumi import openstack:dns/zoneShareV2:ZoneShareV2 share_1 60cbdc69-64f9-49ee-b294-352e71e22827/0e1dae51-aee2-4b44-962f-885bb69f3a5c/eb92139f6c054a878852ac9e8cbe612a
        ```


        :param str resource_name: The name of the resource.
        :param ZoneShareV2Args args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ZoneShareV2Args, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 target_project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 zone_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ZoneShareV2Args.__new__(ZoneShareV2Args)

            __props__.__dict__["project_id"] = project_id
            __props__.__dict__["region"] = region
            if target_project_id is None and not opts.urn:
                raise TypeError("Missing required property 'target_project_id'")
            __props__.__dict__["target_project_id"] = target_project_id
            if zone_id is None and not opts.urn:
                raise TypeError("Missing required property 'zone_id'")
            __props__.__dict__["zone_id"] = zone_id
        super(ZoneShareV2, __self__).__init__(
            'openstack:dns/zoneShareV2:ZoneShareV2',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            project_id: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            target_project_id: Optional[pulumi.Input[_builtins.str]] = None,
            zone_id: Optional[pulumi.Input[_builtins.str]] = None) -> 'ZoneShareV2':
        """
        Get an existing ZoneShareV2 resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] project_id: The ID of the project DNS zone is created for, sets
               `X-Auth-Sudo-Tenant-ID` header (requires an assigned user role in target
               project).
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 DNS client. If
               omitted, the `region` argument of the provider is used. Changing this creates
               a new DNS zone share.
        :param pulumi.Input[_builtins.str] target_project_id: The ID of the target project with which the
               DNS zone will be shared.
        :param pulumi.Input[_builtins.str] zone_id: The ID of the DNS zone to be shared.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ZoneShareV2State.__new__(_ZoneShareV2State)

        __props__.__dict__["project_id"] = project_id
        __props__.__dict__["region"] = region
        __props__.__dict__["target_project_id"] = target_project_id
        __props__.__dict__["zone_id"] = zone_id
        return ZoneShareV2(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the project DNS zone is created for, sets
        `X-Auth-Sudo-Tenant-ID` header (requires an assigned user role in target
        project).
        """
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region in which to obtain the V2 DNS client. If
        omitted, the `region` argument of the provider is used. Changing this creates
        a new DNS zone share.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="targetProjectId")
    def target_project_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the target project with which the
        DNS zone will be shared.
        """
        return pulumi.get(self, "target_project_id")

    @_builtins.property
    @pulumi.getter(name="zoneId")
    def zone_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the DNS zone to be shared.
        """
        return pulumi.get(self, "zone_id")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'GetAggregateV2Result',
    'AwaitableGetAggregateV2Result',
    'get_aggregate_v2',
    'get_aggregate_v2_output',
]

@pulumi.output_type
class GetAggregateV2Result:
    """
    A collection of values returned by getAggregateV2.
    """
    def __init__(__self__, hosts=None, id=None, metadata=None, name=None, region=None, zone=None):
        if hosts and not isinstance(hosts, list):
            raise TypeError("Expected argument 'hosts' to be a list")
        pulumi.set(__self__, "hosts", hosts)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if metadata and not isinstance(metadata, dict):
            raise TypeError("Expected argument 'metadata' to be a dict")
        pulumi.set(__self__, "metadata", metadata)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if zone and not isinstance(zone, str):
            raise TypeError("Expected argument 'zone' to be a str")
        pulumi.set(__self__, "zone", zone)

    @_builtins.property
    @pulumi.getter
    def hosts(self) -> Sequence[_builtins.str]:
        """
        List of Hypervisors contained in the Host Aggregate
        """
        return pulumi.get(self, "hosts")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Mapping[str, _builtins.str]:
        """
        Metadata of the Host Aggregate
        """
        return pulumi.get(self, "metadata")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        See Argument Reference above.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        """
        See Argument Reference above.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def zone(self) -> _builtins.str:
        """
        Availability zone of the Host Aggregate
        """
        return pulumi.get(self, "zone")


class AwaitableGetAggregateV2Result(GetAggregateV2Result):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAggregateV2Result(
            hosts=self.hosts,
            id=self.id,
            metadata=self.metadata,
            name=self.name,
            region=self.region,
            zone=self.zone)


def get_aggregate_v2(hosts: Optional[Sequence[_builtins.str]] = None,
                     metadata: Optional[Mapping[str, _builtins.str]] = None,
                     name: Optional[_builtins.str] = None,
                     region: Optional[_builtins.str] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAggregateV2Result:
    """
    Use this data source to get information about host aggregates
    by name.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    test = openstack.compute.get_aggregate_v2(name="test")
    ```


    :param Sequence[_builtins.str] hosts: List of Hypervisors contained in the Host Aggregate
    :param Mapping[str, _builtins.str] metadata: Metadata of the Host Aggregate
    :param _builtins.str name: The name of the host aggregate.
    :param _builtins.str region: The region in which to obtain the V2 Compute client.
           If omitted, the `region` argument of the provider is used.
    """
    __args__ = dict()
    __args__['hosts'] = hosts
    __args__['metadata'] = metadata
    __args__['name'] = name
    __args__['region'] = region
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('openstack:compute/getAggregateV2:getAggregateV2', __args__, opts=opts, typ=GetAggregateV2Result).value

    return AwaitableGetAggregateV2Result(
        hosts=pulumi.get(__ret__, 'hosts'),
        id=pulumi.get(__ret__, 'id'),
        metadata=pulumi.get(__ret__, 'metadata'),
        name=pulumi.get(__ret__, 'name'),
        region=pulumi.get(__ret__, 'region'),
        zone=pulumi.get(__ret__, 'zone'))
def get_aggregate_v2_output(hosts: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                            metadata: Optional[pulumi.Input[Optional[Mapping[str, _builtins.str]]]] = None,
                            name: Optional[pulumi.Input[_builtins.str]] = None,
                            region: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                            opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAggregateV2Result]:
    """
    Use this data source to get information about host aggregates
    by name.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    test = openstack.compute.get_aggregate_v2(name="test")
    ```


    :param Sequence[_builtins.str] hosts: List of Hypervisors contained in the Host Aggregate
    :param Mapping[str, _builtins.str] metadata: Metadata of the Host Aggregate
    :param _builtins.str name: The name of the host aggregate.
    :param _builtins.str region: The region in which to obtain the V2 Compute client.
           If omitted, the `region` argument of the provider is used.
    """
    __args__ = dict()
    __args__['hosts'] = hosts
    __args__['metadata'] = metadata
    __args__['name'] = name
    __args__['region'] = region
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('openstack:compute/getAggregateV2:getAggregateV2', __args__, opts=opts, typ=GetAggregateV2Result)
    return __ret__.apply(lambda __response__: GetAggregateV2Result(
        hosts=pulumi.get(__response__, 'hosts'),
        id=pulumi.get(__response__, 'id'),
        metadata=pulumi.get(__response__, 'metadata'),
        name=pulumi.get(__response__, 'name'),
        region=pulumi.get(__response__, 'region'),
        zone=pulumi.get(__response__, 'zone')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['VolumeAttachArgs', 'VolumeAttach']

@pulumi.input_type
class VolumeAttachArgs:
    def __init__(__self__, *,
                 instance_id: pulumi.Input[_builtins.str],
                 volume_id: pulumi.Input[_builtins.str],
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']] = None):
        """
        The set of arguments for constructing a VolumeAttach resource.

        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        :param pulumi.Input[_builtins.str] device: The device of the volume attachment (ex: `/dev/vdc`).
               _NOTE_: Being able to specify a device is dependent upon the hypervisor in
               use. There is a chance that the device specified in Terraform will not be
               the same device the hypervisor chose. If this happens, Terraform will wish
               to update the device upon subsequent applying which will cause the volume
               to be detached and reattached indefinitely. Please use with caution.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input['VolumeAttachVendorOptionsArgs'] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        """
        pulumi.set(__self__, "instance_id", instance_id)
        pulumi.set(__self__, "volume_id", volume_id)
        if device is not None:
            pulumi.set(__self__, "device", device)
        if multiattach is not None:
            pulumi.set(__self__, "multiattach", multiattach)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if tag is not None:
            pulumi.set(__self__, "tag", tag)
        if vendor_options is not None:
            pulumi.set(__self__, "vendor_options", vendor_options)

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the Instance to attach the Volume to.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "instance_id", value)

    @_builtins.property
    @pulumi.getter(name="volumeId")
    def volume_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the Volume to attach to an Instance.
        """
        return pulumi.get(self, "volume_id")

    @volume_id.setter
    def volume_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "volume_id", value)

    @_builtins.property
    @pulumi.getter
    def device(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The device of the volume attachment (ex: `/dev/vdc`).
        _NOTE_: Being able to specify a device is dependent upon the hypervisor in
        use. There is a chance that the device specified in Terraform will not be
        the same device the hypervisor chose. If this happens, Terraform will wish
        to update the device upon subsequent applying which will cause the volume
        to be detached and reattached indefinitely. Please use with caution.
        """
        return pulumi.get(self, "device")

    @device.setter
    def device(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "device", value)

    @_builtins.property
    @pulumi.getter
    def multiattach(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Enable attachment of multiattach-capable volumes.
        """
        return pulumi.get(self, "multiattach")

    @multiattach.setter
    def multiattach(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "multiattach", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a volume attachment. If omitted, the
        `region` argument of the provider is used. Changing this creates a
        new volume attachment.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def tag(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Add a device role tag that is applied to the volume when
        attaching it to the VM. Changing this creates a new volume attachment with
        the new tag. Requires microversion >= 2.49.
        """
        return pulumi.get(self, "tag")

    @tag.setter
    def tag(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "tag", value)

    @_builtins.property
    @pulumi.getter(name="vendorOptions")
    def vendor_options(self) -> Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]:
        """
        Map of additional vendor-specific options.
        Supported options are described below.
        """
        return pulumi.get(self, "vendor_options")

    @vendor_options.setter
    def vendor_options(self, value: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]):
        pulumi.set(self, "vendor_options", value)


@pulumi.input_type
class _VolumeAttachState:
    def __init__(__self__, *,
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']] = None,
                 volume_id: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering VolumeAttach resources.

        :param pulumi.Input[_builtins.str] device: The device of the volume attachment (ex: `/dev/vdc`).
               _NOTE_: Being able to specify a device is dependent upon the hypervisor in
               use. There is a chance that the device specified in Terraform will not be
               the same device the hypervisor chose. If this happens, Terraform will wish
               to update the device upon subsequent applying which will cause the volume
               to be detached and reattached indefinitely. Please use with caution.
        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input['VolumeAttachVendorOptionsArgs'] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        """
        if device is not None:
            pulumi.set(__self__, "device", device)
        if instance_id is not None:
            pulumi.set(__self__, "instance_id", instance_id)
        if multiattach is not None:
            pulumi.set(__self__, "multiattach", multiattach)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if tag is not None:
            pulumi.set(__self__, "tag", tag)
        if vendor_options is not None:
            pulumi.set(__self__, "vendor_options", vendor_options)
        if volume_id is not None:
            pulumi.set(__self__, "volume_id", volume_id)

    @_builtins.property
    @pulumi.getter
    def device(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The device of the volume attachment (ex: `/dev/vdc`).
        _NOTE_: Being able to specify a device is dependent upon the hypervisor in
        use. There is a chance that the device specified in Terraform will not be
        the same device the hypervisor chose. If this happens, Terraform will wish
        to update the device upon subsequent applying which will cause the volume
        to be detached and reattached indefinitely. Please use with caution.
        """
        return pulumi.get(self, "device")

    @device.setter
    def device(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "device", value)

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the Instance to attach the Volume to.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "instance_id", value)

    @_builtins.property
    @pulumi.getter
    def multiattach(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Enable attachment of multiattach-capable volumes.
        """
        return pulumi.get(self, "multiattach")

    @multiattach.setter
    def multiattach(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "multiattach", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a volume attachment. If omitted, the
        `region` argument of the provider is used. Changing this creates a
        new volume attachment.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def tag(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Add a device role tag that is applied to the volume when
        attaching it to the VM. Changing this creates a new volume attachment with
        the new tag. Requires microversion >= 2.49.
        """
        return pulumi.get(self, "tag")

    @tag.setter
    def tag(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "tag", value)

    @_builtins.property
    @pulumi.getter(name="vendorOptions")
    def vendor_options(self) -> Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]:
        """
        Map of additional vendor-specific options.
        Supported options are described below.
        """
        return pulumi.get(self, "vendor_options")

    @vendor_options.setter
    def vendor_options(self, value: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]):
        pulumi.set(self, "vendor_options", value)

    @_builtins.property
    @pulumi.getter(name="volumeId")
    def volume_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the Volume to attach to an Instance.
        """
        return pulumi.get(self, "volume_id")

    @volume_id.setter
    def volume_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "volume_id", value)


@pulumi.type_token("openstack:compute/volumeAttach:VolumeAttach")
class VolumeAttach(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']]] = None,
                 volume_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Attaches a Block Storage Volume to an Instance using the OpenStack
        Compute (Nova) v2 API.

        ## Example Usage

        ### Basic attachment of a single volume to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1)
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id)
        ```

        ### Attaching multiple volumes to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attachments = []
        for range in [{"value": i} for i in range(0, 2)]:
            attachments.append(openstack.compute.VolumeAttach(f"attachments-{range['value']}",
                instance_id=instance1.id,
                volume_id=volumes[range["value"]].id))
        pulumi.export("volumeDevices", [__item.device for __item in attachments])
        ```

        Note that the above example will not guarantee that the volumes are attached in
        a deterministic manner. The volumes will be attached in a seemingly random
        order.

        If you want to ensure that the volumes are attached in a given order, create
        explicit dependencies between the volumes, such as:

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attach1 = openstack.compute.VolumeAttach("attach_1",
            instance_id=instance1.id,
            volume_id=volumes[0].id)
        attach2 = openstack.compute.VolumeAttach("attach_2",
            instance_id=instance1.id,
            volume_id=volumes[1].id,
            opts = pulumi.ResourceOptions(depends_on=[attach1]))
        pulumi.export("volumeDevices", [__item["device"] for __item in attachments])
        ```

        ### Using Multiattach-enabled volumes

        Multiattach Volumes are dependent upon your OpenStack cloud and not all
        clouds support multiattach. Multiattach volumes require a volume_type that has [multiattach enabled](https://docs.openstack.org/cinder/latest/admin/volume-multiattach.html#multiattach-volume-type).

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1,
            volume_type="multiattach")
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        instance2 = openstack.compute.Instance("instance_2",
            name="instance_2",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id,
            multiattach=True)
        va2 = openstack.compute.VolumeAttach("va_2",
            instance_id=instance2.id,
            volume_id=volume1.id,
            multiattach=True,
            opts = pulumi.ResourceOptions(depends_on=[va1]))
        ```

        It is recommended to use `depends_on` for the attach resources
        to enforce the volume attachments to happen one at a time.

        ## Import

        Volume Attachments can be imported using the Instance ID and Volume ID
        separated by a slash, e.g.

        ```sh
        $ pulumi import openstack:compute/volumeAttach:VolumeAttach va_1 89c60255-9bd6-460c-822a-e2b959ede9d2/45670584-225f-46c3-b33e-6707b589b666
        ```


        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] device: The device of the volume attachment (ex: `/dev/vdc`).
               _NOTE_: Being able to specify a device is dependent upon the hypervisor in
               use. There is a chance that the device specified in Terraform will not be
               the same device the hypervisor chose. If this happens, Terraform will wish
               to update the device upon subsequent applying which will cause the volume
               to be detached and reattached indefinitely. Please use with caution.
        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: VolumeAttachArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Attaches a Block Storage Volume to an Instance using the OpenStack
        Compute (Nova) v2 API.

        ## Example Usage

        ### Basic attachment of a single volume to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1)
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id)
        ```

        ### Attaching multiple volumes to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attachments = []
        for range in [{"value": i} for i in range(0, 2)]:
            attachments.append(openstack.compute.VolumeAttach(f"attachments-{range['value']}",
                instance_id=instance1.id,
                volume_id=volumes[range["value"]].id))
        pulumi.export("volumeDevices", [__item.device for __item in attachments])
        ```

        Note that the above example will not guarantee that the volumes are attached in
        a deterministic manner. The volumes will be attached in a seemingly random
        order.

        If you want to ensure that the volumes are attached in a given order, create
        explicit dependencies between the volumes, such as:

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attach1 = openstack.compute.VolumeAttach("attach_1",
            instance_id=instance1.id,
            volume_id=volumes[0].id)
        attach2 = openstack.compute.VolumeAttach("attach_2",
            instance_id=instance1.id,
            volume_id=volumes[1].id,
            opts = pulumi.ResourceOptions(depends_on=[attach1]))
        pulumi.export("volumeDevices", [__item["device"] for __item in attachments])
        ```

        ### Using Multiattach-enabled volumes

        Multiattach Volumes are dependent upon your OpenStack cloud and not all
        clouds support multiattach. Multiattach volumes require a volume_type that has [multiattach enabled](https://docs.openstack.org/cinder/latest/admin/volume-multiattach.html#multiattach-volume-type).

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1,
            volume_type="multiattach")
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        instance2 = openstack.compute.Instance("instance_2",
            name="instance_2",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id,
            multiattach=True)
        va2 = openstack.compute.VolumeAttach("va_2",
            instance_id=instance2.id,
            volume_id=volume1.id,
            multiattach=True,
            opts = pulumi.ResourceOptions(depends_on=[va1]))
        ```

        It is recommended to use `depends_on` for the attach resources
        to enforce the volume attachments to happen one at a time.

        ## Import

        Volume Attachments can be imported using the Instance ID and Volume ID
        separated by a slash, e.g.

        ```sh
        $ pulumi import openstack:compute/volumeAttach:VolumeAttach va_1 89c60255-9bd6-460c-822a-e2b959ede9d2/45670584-225f-46c3-b33e-6707b589b666
        ```


        :param str resource_name: The name of the resource.
        :param VolumeAttachArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(VolumeAttachArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']]] = None,
                 volume_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = VolumeAttachArgs.__new__(VolumeAttachArgs)

            __props__.__dict__["device"] = device
            if instance_id is None and not opts.urn:
                raise TypeError("Missing required property 'instance_id'")
            __props__.__dict__["instance_id"] = instance_id
            __props__.__dict__["multiattach"] = multiattach
            __props__.__dict__["region"] = region
            __props__.__dict__["tag"] = tag
            __props__.__dict__["vendor_options"] = vendor_options
            if volume_id is None and not opts.urn:
                raise TypeError("Missing required property 'volume_id'")
            __props__.__dict__["volume_id"] = volume_id
        super(VolumeAttach, __self__).__init__(
            'openstack:compute/volumeAttach:VolumeAttach',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            device: Optional[pulumi.Input[_builtins.str]] = None,
            instance_id: Optional[pulumi.Input[_builtins.str]] = None,
            multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            tag: Optional[pulumi.Input[_builtins.str]] = None,
            vendor_options: Optional[pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']]] = None,
            volume_id: Optional[pulumi.Input[_builtins.str]] = None) -> 'VolumeAttach':
        """
        Get an existing VolumeAttach resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] device: The device of the volume attachment (ex: `/dev/vdc`).
               _NOTE_: Being able to specify a device is dependent upon the hypervisor in
               use. There is a chance that the device specified in Terraform will not be
               the same device the hypervisor chose. If this happens, Terraform will wish
               to update the device upon subsequent applying which will cause the volume
               to be detached and reattached indefinitely. Please use with caution.
        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _VolumeAttachState.__new__(_VolumeAttachState)

        __props__.__dict__["device"] = device
        __props__.__dict__["instance_id"] = instance_id
        __props__.__dict__["multiattach"] = multiattach
        __props__.__dict__["region"] = region
        __props__.__dict__["tag"] = tag
        __props__.__dict__["vendor_options"] = vendor_options
        __props__.__dict__["volume_id"] = volume_id
        return VolumeAttach(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter
    def device(self) -> pulumi.Output[_builtins.str]:
        """
        The device of the volume attachment (ex: `/dev/vdc`).
        _NOTE_: Being able to specify a device is dependent upon the hypervisor in
        use. There is a chance that the device specified in Terraform will not be
        the same device the hypervisor chose. If this happens, Terraform will wish
        to update the device upon subsequent applying which will cause the volume
        to be detached and reattached indefinitely. Please use with caution.
        """
        return pulumi.get(self, "device")

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the Instance to attach the Volume to.
        """
        return pulumi.get(self, "instance_id")

    @_builtins.property
    @pulumi.getter
    def multiattach(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        Enable attachment of multiattach-capable volumes.
        """
        return pulumi.get(self, "multiattach")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a volume attachment. If omitted, the
        `region` argument of the provider is used. Changing this creates a
        new volume attachment.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def tag(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Add a device role tag that is applied to the volume when
        attaching it to the VM. Changing this creates a new volume attachment with
        the new tag. Requires microversion >= 2.49.
        """
        return pulumi.get(self, "tag")

    @_builtins.property
    @pulumi.getter(name="vendorOptions")
    def vendor_options(self) -> pulumi.Output[Optional['outputs.VolumeAttachVendorOptions']]:
        """
        Map of additional vendor-specific options.
        Supported options are described below.
        """
        return pulumi.get(self, "vendor_options")

    @_builtins.property
    @pulumi.getter(name="volumeId")
    def volume_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the Volume to attach to an Instance.
        """
        return pulumi.get(self, "volume_id")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['OrderV1Args', 'OrderV1']

@pulumi.input_type
class OrderV1Args:
    def __init__(__self__, *,
                 meta: pulumi.Input['OrderV1MetaArgs'],
                 type: pulumi.Input[_builtins.str],
                 region: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a OrderV1 resource.

        :param pulumi.Input['OrderV1MetaArgs'] meta: Dictionary containing the order metadata used to generate the order. The structure is described below.
        :param pulumi.Input[_builtins.str] type: The type of key to be generated. Must be one of `asymmetric`, `key`.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V1 KeyManager client.
               A KeyManager client is needed to create a order. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               V1 order.
        """
        pulumi.set(__self__, "meta", meta)
        pulumi.set(__self__, "type", type)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter
    def meta(self) -> pulumi.Input['OrderV1MetaArgs']:
        """
        Dictionary containing the order metadata used to generate the order. The structure is described below.
        """
        return pulumi.get(self, "meta")

    @meta.setter
    def meta(self, value: pulumi.Input['OrderV1MetaArgs']):
        pulumi.set(self, "meta", value)

    @_builtins.property
    @pulumi.getter
    def type(self) -> pulumi.Input[_builtins.str]:
        """
        The type of key to be generated. Must be one of `asymmetric`, `key`.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V1 KeyManager client.
        A KeyManager client is needed to create a order. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        V1 order.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class _OrderV1State:
    def __init__(__self__, *,
                 container_ref: Optional[pulumi.Input[_builtins.str]] = None,
                 created: Optional[pulumi.Input[_builtins.str]] = None,
                 creator_id: Optional[pulumi.Input[_builtins.str]] = None,
                 meta: Optional[pulumi.Input['OrderV1MetaArgs']] = None,
                 order_ref: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 secret_ref: Optional[pulumi.Input[_builtins.str]] = None,
                 status: Optional[pulumi.Input[_builtins.str]] = None,
                 sub_status: Optional[pulumi.Input[_builtins.str]] = None,
                 sub_status_message: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 updated: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering OrderV1 resources.

        :param pulumi.Input[_builtins.str] container_ref: The container reference / where to find the container.
        :param pulumi.Input[_builtins.str] created: The date the order was created.
        :param pulumi.Input[_builtins.str] creator_id: The creator of the order.
        :param pulumi.Input['OrderV1MetaArgs'] meta: Dictionary containing the order metadata used to generate the order. The structure is described below.
        :param pulumi.Input[_builtins.str] order_ref: The order reference / where to find the order.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V1 KeyManager client.
               A KeyManager client is needed to create a order. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               V1 order.
        :param pulumi.Input[_builtins.str] secret_ref: The secret reference / where to find the secret.
        :param pulumi.Input[_builtins.str] status: The status of the order.
        :param pulumi.Input[_builtins.str] sub_status: The sub status of the order.
        :param pulumi.Input[_builtins.str] sub_status_message: The sub status message of the order.
        :param pulumi.Input[_builtins.str] type: The type of key to be generated. Must be one of `asymmetric`, `key`.
        :param pulumi.Input[_builtins.str] updated: The date the order was last updated.
        """
        if container_ref is not None:
            pulumi.set(__self__, "container_ref", container_ref)
        if created is not None:
            pulumi.set(__self__, "created", created)
        if creator_id is not None:
            pulumi.set(__self__, "creator_id", creator_id)
        if meta is not None:
            pulumi.set(__self__, "meta", meta)
        if order_ref is not None:
            pulumi.set(__self__, "order_ref", order_ref)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if secret_ref is not None:
            pulumi.set(__self__, "secret_ref", secret_ref)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if sub_status is not None:
            pulumi.set(__self__, "sub_status", sub_status)
        if sub_status_message is not None:
            pulumi.set(__self__, "sub_status_message", sub_status_message)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if updated is not None:
            pulumi.set(__self__, "updated", updated)

    @_builtins.property
    @pulumi.getter(name="containerRef")
    def container_ref(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The container reference / where to find the container.
        """
        return pulumi.get(self, "container_ref")

    @container_ref.setter
    def container_ref(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "container_ref", value)

    @_builtins.property
    @pulumi.getter
    def created(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The date the order was created.
        """
        return pulumi.get(self, "created")

    @created.setter
    def created(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "created", value)

    @_builtins.property
    @pulumi.getter(name="creatorId")
    def creator_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The creator of the order.
        """
        return pulumi.get(self, "creator_id")

    @creator_id.setter
    def creator_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "creator_id", value)

    @_builtins.property
    @pulumi.getter
    def meta(self) -> Optional[pulumi.Input['OrderV1MetaArgs']]:
        """
        Dictionary containing the order metadata used to generate the order. The structure is described below.
        """
        return pulumi.get(self, "meta")

    @meta.setter
    def meta(self, value: Optional[pulumi.Input['OrderV1MetaArgs']]):
        pulumi.set(self, "meta", value)

    @_builtins.property
    @pulumi.getter(name="orderRef")
    def order_ref(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The order reference / where to find the order.
        """
        return pulumi.get(self, "order_ref")

    @order_ref.setter
    def order_ref(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "order_ref", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V1 KeyManager client.
        A KeyManager client is needed to create a order. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        V1 order.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="secretRef")
    def secret_ref(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The secret reference / where to find the secret.
        """
        return pulumi.get(self, "secret_ref")

    @secret_ref.setter
    def secret_ref(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "secret_ref", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The status of the order.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "status", value)

    @_builtins.property
    @pulumi.getter(name="subStatus")
    def sub_status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The sub status of the order.
        """
        return pulumi.get(self, "sub_status")

    @sub_status.setter
    def sub_status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "sub_status", value)

    @_builtins.property
    @pulumi.getter(name="subStatusMessage")
    def sub_status_message(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The sub status message of the order.
        """
        return pulumi.get(self, "sub_status_message")

    @sub_status_message.setter
    def sub_status_message(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "sub_status_message", value)

    @_builtins.property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The type of key to be generated. Must be one of `asymmetric`, `key`.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def updated(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The date the order was last updated.
        """
        return pulumi.get(self, "updated")

    @updated.setter
    def updated(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "updated", value)


@pulumi.type_token("openstack:keymanager/orderV1:OrderV1")
class OrderV1(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 meta: Optional[pulumi.Input[Union['OrderV1MetaArgs', 'OrderV1MetaArgsDict']]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Manages a V1 Barbican order resource within OpenStack.

        ## Example Usage

        ### Symmetric key order

        ```python
        import pulumi
        import pulumi_openstack as openstack

        order1 = openstack.keymanager.OrderV1("order_1",
            type="key",
            meta={
                "algorithm": "aes",
                "bit_length": 256,
                "name": "mysecret",
                "mode": "cbc",
            })
        ```

        ### Asymmetric key pair order

        ```python
        import pulumi
        import pulumi_openstack as openstack

        order1 = openstack.keymanager.OrderV1("order_1",
            type="asymmetric",
            meta={
                "algorithm": "rsa",
                "bit_length": 4096,
                "name": "mysecret",
            })
        ```

        ## Import

        Orders can be imported using the order id (the last part of the order reference), e.g.:

        ```sh
        $ pulumi import openstack:keymanager/orderV1:OrderV1 order_1 0c6cd26a-c012-4d7b-8034-057c0f1c2953
        ```


        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Union['OrderV1MetaArgs', 'OrderV1MetaArgsDict']] meta: Dictionary containing the order metadata used to generate the order. The structure is described below.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V1 KeyManager client.
               A KeyManager client is needed to create a order. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               V1 order.
        :param pulumi.Input[_builtins.str] type: The type of key to be generated. Must be one of `asymmetric`, `key`.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: OrderV1Args,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a V1 Barbican order resource within OpenStack.

        ## Example Usage

        ### Symmetric key order

        ```python
        import pulumi
        import pulumi_openstack as openstack

        order1 = openstack.keymanager.OrderV1("order_1",
            type="key",
            meta={
                "algorithm": "aes",
                "bit_length": 256,
                "name": "mysecret",
                "mode": "cbc",
            })
        ```

        ### Asymmetric key pair order

        ```python
        import pulumi
        import pulumi_openstack as openstack

        order1 = openstack.keymanager.OrderV1("order_1",
            type="asymmetric",
            meta={
                "algorithm": "rsa",
                "bit_length": 4096,
                "name": "mysecret",
            })
        ```

        ## Import

        Orders can be imported using the order id (the last part of the order reference), e.g.:

        ```sh
        $ pulumi import openstack:keymanager/orderV1:OrderV1 order_1 0c6cd26a-c012-4d7b-8034-057c0f1c2953
        ```


        :param str resource_name: The name of the resource.
        :param OrderV1Args args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(OrderV1Args, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 meta: Optional[pulumi.Input[Union['OrderV1MetaArgs', 'OrderV1MetaArgsDict']]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = OrderV1Args.__new__(OrderV1Args)

            if meta is None and not opts.urn:
                raise TypeError("Missing required property 'meta'")
            __props__.__dict__["meta"] = meta
            __props__.__dict__["region"] = region
            if type is None and not opts.urn:
                raise TypeError("Missing required property 'type'")
            __props__.__dict__["type"] = type
            __props__.__dict__["container_ref"] = None
            __props__.__dict__["created"] = None
            __props__.__dict__["creator_id"] = None
            __props__.__dict__["order_ref"] = None
            __props__.__dict__["secret_ref"] = None
            __props__.__dict__["status"] = None
            __props__.__dict__["sub_status"] = None
            __props__.__dict__["sub_status_message"] = None
            __props__.__dict__["updated"] = None
        super(OrderV1, __self__).__init__(
            'openstack:keymanager/orderV1:OrderV1',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            container_ref: Optional[pulumi.Input[_builtins.str]] = None,
            created: Optional[pulumi.Input[_builtins.str]] = None,
            creator_id: Optional[pulumi.Input[_builtins.str]] = None,
            meta: Optional[pulumi.Input[Union['OrderV1MetaArgs', 'OrderV1MetaArgsDict']]] = None,
            order_ref: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            secret_ref: Optional[pulumi.Input[_builtins.str]] = None,
            status: Optional[pulumi.Input[_builtins.str]] = None,
            sub_status: Optional[pulumi.Input[_builtins.str]] = None,
            sub_status_message: Optional[pulumi.Input[_builtins.str]] = None,
            type: Optional[pulumi.Input[_builtins.str]] = None,
            updated: Optional[pulumi.Input[_builtins.str]] = None) -> 'OrderV1':
        """
        Get an existing OrderV1 resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] container_ref: The container reference / where to find the container.
        :param pulumi.Input[_builtins.str] created: The date the order was created.
        :param pulumi.Input[_builtins.str] creator_id: The creator of the order.
        :param pulumi.Input[Union['OrderV1MetaArgs', 'OrderV1MetaArgsDict']] meta: Dictionary containing the order metadata used to generate the order. The structure is described below.
        :param pulumi.Input[_builtins.str] order_ref: The order reference / where to find the order.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V1 KeyManager client.
               A KeyManager client is needed to create a order. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               V1 order.
        :param pulumi.Input[_builtins.str] secret_ref: The secret reference / where to find the secret.
        :param pulumi.Input[_builtins.str] status: The status of the order.
        :param pulumi.Input[_builtins.str] sub_status: The sub status of the order.
        :param pulumi.Input[_builtins.str] sub_status_message: The sub status message of the order.
        :param pulumi.Input[_builtins.str] type: The type of key to be generated. Must be one of `asymmetric`, `key`.
        :param pulumi.Input[_builtins.str] updated: The date the order was last updated.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _OrderV1State.__new__(_OrderV1State)

        __props__.__dict__["container_ref"] = container_ref
        __props__.__dict__["created"] = created
        __props__.__dict__["creator_id"] = creator_id
        __props__.__dict__["meta"] = meta
        __props__.__dict__["order_ref"] = order_ref
        __props__.__dict__["region"] = region
        __props__.__dict__["secret_ref"] = secret_ref
        __props__.__dict__["status"] = status
        __props__.__dict__["sub_status"] = sub_status
        __props__.__dict__["sub_status_message"] = sub_status_message
        __props__.__dict__["type"] = type
        __props__.__dict__["updated"] = updated
        return OrderV1(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="containerRef")
    def container_ref(self) -> pulumi.Output[_builtins.str]:
        """
        The container reference / where to find the container.
        """
        return pulumi.get(self, "container_ref")

    @_builtins.property
    @pulumi.getter
    def created(self) -> pulumi.Output[_builtins.str]:
        """
        The date the order was created.
        """
        return pulumi.get(self, "created")

    @_builtins.property
    @pulumi.getter(name="creatorId")
    def creator_id(self) -> pulumi.Output[_builtins.str]:
        """
        The creator of the order.
        """
        return pulumi.get(self, "creator_id")

    @_builtins.property
    @pulumi.getter
    def meta(self) -> pulumi.Output['outputs.OrderV1Meta']:
        """
        Dictionary containing the order metadata used to generate the order. The structure is described below.
        """
        return pulumi.get(self, "meta")

    @_builtins.property
    @pulumi.getter(name="orderRef")
    def order_ref(self) -> pulumi.Output[_builtins.str]:
        """
        The order reference / where to find the order.
        """
        return pulumi.get(self, "order_ref")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region in which to obtain the V1 KeyManager client.
        A KeyManager client is needed to create a order. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        V1 order.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="secretRef")
    def secret_ref(self) -> pulumi.Output[_builtins.str]:
        """
        The secret reference / where to find the secret.
        """
        return pulumi.get(self, "secret_ref")

    @_builtins.property
    @pulumi.getter
    def status(self) -> pulumi.Output[_builtins.str]:
        """
        The status of the order.
        """
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter(name="subStatus")
    def sub_status(self) -> pulumi.Output[_builtins.str]:
        """
        The sub status of the order.
        """
        return pulumi.get(self, "sub_status")

    @_builtins.property
    @pulumi.getter(name="subStatusMessage")
    def sub_status_message(self) -> pulumi.Output[_builtins.str]:
        """
        The sub status message of the order.
        """
        return pulumi.get(self, "sub_status_message")

    @_builtins.property
    @pulumi.getter
    def type(self) -> pulumi.Output[_builtins.str]:
        """
        The type of key to be generated. Must be one of `asymmetric`, `key`.
        """
        return pulumi.get(self, "type")

    @_builtins.property
    @pulumi.getter
    def updated(self) -> pulumi.Output[_builtins.str]:
        """
        The date the order was last updated.
        """
        return pulumi.get(self, "updated")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'ConfigurationConfigurationArgs',
    'ConfigurationConfigurationArgsDict',
    'ConfigurationDatastoreArgs',
    'ConfigurationDatastoreArgsDict',
    'InstanceDatabaseArgs',
    'InstanceDatabaseArgsDict',
    'InstanceDatastoreArgs',
    'InstanceDatastoreArgsDict',
    'InstanceNetworkArgs',
    'InstanceNetworkArgsDict',
    'InstanceUserArgs',
    'InstanceUserArgsDict',
]

class ConfigurationConfigurationArgsDict(TypedDict):
    name: pulumi.Input[_builtins.str]
    """
    Configuration parameter name. Changing this creates a new resource.
    """
    value: pulumi.Input[_builtins.str]
    """
    Configuration parameter value. Changing this creates a new resource.
    """
    string_type: NotRequired[pulumi.Input[_builtins.bool]]
    """
    Whether or not to store configuration parameter value as string. Changing this creates a new resource. See the below note for more information.
    """

@pulumi.input_type
class ConfigurationConfigurationArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[_builtins.str],
                 value: pulumi.Input[_builtins.str],
                 string_type: Optional[pulumi.Input[_builtins.bool]] = None):
        """
        :param pulumi.Input[_builtins.str] name: Configuration parameter name. Changing this creates a new resource.
        :param pulumi.Input[_builtins.str] value: Configuration parameter value. Changing this creates a new resource.
        :param pulumi.Input[_builtins.bool] string_type: Whether or not to store configuration parameter value as string. Changing this creates a new resource. See the below note for more information.
        """
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "value", value)
        if string_type is not None:
            pulumi.set(__self__, "string_type", string_type)

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Input[_builtins.str]:
        """
        Configuration parameter name. Changing this creates a new resource.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def value(self) -> pulumi.Input[_builtins.str]:
        """
        Configuration parameter value. Changing this creates a new resource.
        """
        return pulumi.get(self, "value")

    @value.setter
    def value(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "value", value)

    @_builtins.property
    @pulumi.getter(name="stringType")
    def string_type(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Whether or not to store configuration parameter value as string. Changing this creates a new resource. See the below note for more information.
        """
        return pulumi.get(self, "string_type")

    @string_type.setter
    def string_type(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "string_type", value)


class ConfigurationDatastoreArgsDict(TypedDict):
    type: pulumi.Input[_builtins.str]
    """
    Database engine type to be used with this configuration. Changing this creates a new resource.
    """
    version: pulumi.Input[_builtins.str]
    """
    Version of database engine type to be used with this configuration. Changing this creates a new resource.
    """

@pulumi.input_type
class ConfigurationDatastoreArgs:
    def __init__(__self__, *,
                 type: pulumi.Input[_builtins.str],
                 version: pulumi.Input[_builtins.str]):
        """
        :param pulumi.Input[_builtins.str] type: Database engine type to be used with this configuration. Changing this creates a new resource.
        :param pulumi.Input[_builtins.str] version: Version of database engine type to be used with this configuration. Changing this creates a new resource.
        """
        pulumi.set(__self__, "type", type)
        pulumi.set(__self__, "version", version)

    @_builtins.property
    @pulumi.getter
    def type(self) -> pulumi.Input[_builtins.str]:
        """
        Database engine type to be used with this configuration. Changing this creates a new resource.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def version(self) -> pulumi.Input[_builtins.str]:
        """
        Version of database engine type to be used with this configuration. Changing this creates a new resource.
        """
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "version", value)


class InstanceDatabaseArgsDict(TypedDict):
    name: pulumi.Input[_builtins.str]
    """
    Database to be created on new instance. Changing this creates a
    new instance.
    """
    charset: NotRequired[pulumi.Input[_builtins.str]]
    """
    Database character set. Changing this creates a
    new instance.
    """
    collate: NotRequired[pulumi.Input[_builtins.str]]
    """
    Database collation. Changing this creates a new instance.
    """

@pulumi.input_type
class InstanceDatabaseArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[_builtins.str],
                 charset: Optional[pulumi.Input[_builtins.str]] = None,
                 collate: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] name: Database to be created on new instance. Changing this creates a
               new instance.
        :param pulumi.Input[_builtins.str] charset: Database character set. Changing this creates a
               new instance.
        :param pulumi.Input[_builtins.str] collate: Database collation. Changing this creates a new instance.
        """
        pulumi.set(__self__, "name", name)
        if charset is not None:
            pulumi.set(__self__, "charset", charset)
        if collate is not None:
            pulumi.set(__self__, "collate", collate)

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Input[_builtins.str]:
        """
        Database to be created on new instance. Changing this creates a
        new instance.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def charset(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Database character set. Changing this creates a
        new instance.
        """
        return pulumi.get(self, "charset")

    @charset.setter
    def charset(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "charset", value)

    @_builtins.property
    @pulumi.getter
    def collate(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Database collation. Changing this creates a new instance.
        """
        return pulumi.get(self, "collate")

    @collate.setter
    def collate(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "collate", value)


class InstanceDatastoreArgsDict(TypedDict):
    type: pulumi.Input[_builtins.str]
    """
    Database engine type to be used in new instance. Changing this
    creates a new instance.
    """
    version: pulumi.Input[_builtins.str]
    """
    Version of database engine type to be used in new instance.
    Changing this creates a new instance.
    """

@pulumi.input_type
class InstanceDatastoreArgs:
    def __init__(__self__, *,
                 type: pulumi.Input[_builtins.str],
                 version: pulumi.Input[_builtins.str]):
        """
        :param pulumi.Input[_builtins.str] type: Database engine type to be used in new instance. Changing this
               creates a new instance.
        :param pulumi.Input[_builtins.str] version: Version of database engine type to be used in new instance.
               Changing this creates a new instance.
        """
        pulumi.set(__self__, "type", type)
        pulumi.set(__self__, "version", version)

    @_builtins.property
    @pulumi.getter
    def type(self) -> pulumi.Input[_builtins.str]:
        """
        Database engine type to be used in new instance. Changing this
        creates a new instance.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def version(self) -> pulumi.Input[_builtins.str]:
        """
        Version of database engine type to be used in new instance.
        Changing this creates a new instance.
        """
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "version", value)


class InstanceNetworkArgsDict(TypedDict):
    fixed_ip_v4: NotRequired[pulumi.Input[_builtins.str]]
    """
    Specifies a fixed IPv4 address to be used on this
    network. Changing this creates a new instance.
    """
    fixed_ip_v6: NotRequired[pulumi.Input[_builtins.str]]
    """
    Specifies a fixed IPv6 address to be used on this
    network. Changing this creates a new instance.
    """
    port: NotRequired[pulumi.Input[_builtins.str]]
    """
    The port UUID of a
    network to attach to the instance. Changing this creates a new instance.
    """
    uuid: NotRequired[pulumi.Input[_builtins.str]]
    """
    The network UUID to
    attach to the instance. Changing this creates a new instance.
    """

@pulumi.input_type
class InstanceNetworkArgs:
    def __init__(__self__, *,
                 fixed_ip_v4: Optional[pulumi.Input[_builtins.str]] = None,
                 fixed_ip_v6: Optional[pulumi.Input[_builtins.str]] = None,
                 port: Optional[pulumi.Input[_builtins.str]] = None,
                 uuid: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] fixed_ip_v4: Specifies a fixed IPv4 address to be used on this
               network. Changing this creates a new instance.
        :param pulumi.Input[_builtins.str] fixed_ip_v6: Specifies a fixed IPv6 address to be used on this
               network. Changing this creates a new instance.
        :param pulumi.Input[_builtins.str] port: The port UUID of a
               network to attach to the instance. Changing this creates a new instance.
        :param pulumi.Input[_builtins.str] uuid: The network UUID to
               attach to the instance. Changing this creates a new instance.
        """
        if fixed_ip_v4 is not None:
            pulumi.set(__self__, "fixed_ip_v4", fixed_ip_v4)
        if fixed_ip_v6 is not None:
            pulumi.set(__self__, "fixed_ip_v6", fixed_ip_v6)
        if port is not None:
            pulumi.set(__self__, "port", port)
        if uuid is not None:
            pulumi.set(__self__, "uuid", uuid)

    @_builtins.property
    @pulumi.getter(name="fixedIpV4")
    def fixed_ip_v4(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies a fixed IPv4 address to be used on this
        network. Changing this creates a new instance.
        """
        return pulumi.get(self, "fixed_ip_v4")

    @fixed_ip_v4.setter
    def fixed_ip_v4(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "fixed_ip_v4", value)

    @_builtins.property
    @pulumi.getter(name="fixedIpV6")
    def fixed_ip_v6(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies a fixed IPv6 address to be used on this
        network. Changing this creates a new instance.
        """
        return pulumi.get(self, "fixed_ip_v6")

    @fixed_ip_v6.setter
    def fixed_ip_v6(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "fixed_ip_v6", value)

    @_builtins.property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The port UUID of a
        network to attach to the instance. Changing this creates a new instance.
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "port", value)

    @_builtins.property
    @pulumi.getter
    def uuid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The network UUID to
        attach to the instance. Changing this creates a new instance.
        """
        return pulumi.get(self, "uuid")

    @uuid.setter
    def uuid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "uuid", value)


class InstanceUserArgsDict(TypedDict):
    name: pulumi.Input[_builtins.str]
    """
    Username to be created on new instance. Changing this creates a
    new instance.
    """
    databases: NotRequired[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]
    """
    A list of databases that user will have access to. If not specified,
    user has access to all databases on th einstance. Changing this creates a new instance.
    """
    host: NotRequired[pulumi.Input[_builtins.str]]
    """
    An ip address or % sign indicating what ip addresses can connect with
    this user credentials. Changing this creates a new instance.
    """
    password: NotRequired[pulumi.Input[_builtins.str]]
    """
    User's password. Changing this creates a
    new instance.
    """

@pulumi.input_type
class InstanceUserArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[_builtins.str],
                 databases: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 host: Optional[pulumi.Input[_builtins.str]] = None,
                 password: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] name: Username to be created on new instance. Changing this creates a
               new instance.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] databases: A list of databases that user will have access to. If not specified,
               user has access to all databases on th einstance. Changing this creates a new instance.
        :param pulumi.Input[_builtins.str] host: An ip address or % sign indicating what ip addresses can connect with
               this user credentials. Changing this creates a new instance.
        :param pulumi.Input[_builtins.str] password: User's password. Changing this creates a
               new instance.
        """
        pulumi.set(__self__, "name", name)
        if databases is not None:
            pulumi.set(__self__, "databases", databases)
        if host is not None:
            pulumi.set(__self__, "host", host)
        if password is not None:
            pulumi.set(__self__, "password", password)

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Input[_builtins.str]:
        """
        Username to be created on new instance. Changing this creates a
        new instance.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def databases(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of databases that user will have access to. If not specified,
        user has access to all databases on th einstance. Changing this creates a new instance.
        """
        return pulumi.get(self, "databases")

    @databases.setter
    def databases(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "databases", value)

    @_builtins.property
    @pulumi.getter
    def host(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        An ip address or % sign indicating what ip addresses can connect with
        this user credentials. Changing this creates a new instance.
        """
        return pulumi.get(self, "host")

    @host.setter
    def host(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "host", value)

    @_builtins.property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        User's password. Changing this creates a
        new instance.
        """
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "password", value)





# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'GetSubnetResult',
    'AwaitableGetSubnetResult',
    'get_subnet',
    'get_subnet_output',
]

@pulumi.output_type
class GetSubnetResult:
    """
    A collection of values returned by getSubnet.
    """
    def __init__(__self__, all_tags=None, allocation_pools=None, cidr=None, description=None, dhcp_enabled=None, dns_nameservers=None, dns_publish_fixed_ip=None, enable_dhcp=None, gateway_ip=None, host_routes=None, id=None, ip_version=None, ipv6_address_mode=None, ipv6_ra_mode=None, name=None, network_id=None, region=None, segment_id=None, service_types=None, subnet_id=None, subnetpool_id=None, tags=None, tenant_id=None):
        if all_tags and not isinstance(all_tags, list):
            raise TypeError("Expected argument 'all_tags' to be a list")
        pulumi.set(__self__, "all_tags", all_tags)
        if allocation_pools and not isinstance(allocation_pools, list):
            raise TypeError("Expected argument 'allocation_pools' to be a list")
        pulumi.set(__self__, "allocation_pools", allocation_pools)
        if cidr and not isinstance(cidr, str):
            raise TypeError("Expected argument 'cidr' to be a str")
        pulumi.set(__self__, "cidr", cidr)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if dhcp_enabled and not isinstance(dhcp_enabled, bool):
            raise TypeError("Expected argument 'dhcp_enabled' to be a bool")
        pulumi.set(__self__, "dhcp_enabled", dhcp_enabled)
        if dns_nameservers and not isinstance(dns_nameservers, list):
            raise TypeError("Expected argument 'dns_nameservers' to be a list")
        pulumi.set(__self__, "dns_nameservers", dns_nameservers)
        if dns_publish_fixed_ip and not isinstance(dns_publish_fixed_ip, bool):
            raise TypeError("Expected argument 'dns_publish_fixed_ip' to be a bool")
        pulumi.set(__self__, "dns_publish_fixed_ip", dns_publish_fixed_ip)
        if enable_dhcp and not isinstance(enable_dhcp, bool):
            raise TypeError("Expected argument 'enable_dhcp' to be a bool")
        pulumi.set(__self__, "enable_dhcp", enable_dhcp)
        if gateway_ip and not isinstance(gateway_ip, str):
            raise TypeError("Expected argument 'gateway_ip' to be a str")
        pulumi.set(__self__, "gateway_ip", gateway_ip)
        if host_routes and not isinstance(host_routes, list):
            raise TypeError("Expected argument 'host_routes' to be a list")
        pulumi.set(__self__, "host_routes", host_routes)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip_version and not isinstance(ip_version, int):
            raise TypeError("Expected argument 'ip_version' to be a int")
        pulumi.set(__self__, "ip_version", ip_version)
        if ipv6_address_mode and not isinstance(ipv6_address_mode, str):
            raise TypeError("Expected argument 'ipv6_address_mode' to be a str")
        pulumi.set(__self__, "ipv6_address_mode", ipv6_address_mode)
        if ipv6_ra_mode and not isinstance(ipv6_ra_mode, str):
            raise TypeError("Expected argument 'ipv6_ra_mode' to be a str")
        pulumi.set(__self__, "ipv6_ra_mode", ipv6_ra_mode)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if network_id and not isinstance(network_id, str):
            raise TypeError("Expected argument 'network_id' to be a str")
        pulumi.set(__self__, "network_id", network_id)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if segment_id and not isinstance(segment_id, str):
            raise TypeError("Expected argument 'segment_id' to be a str")
        pulumi.set(__self__, "segment_id", segment_id)
        if service_types and not isinstance(service_types, list):
            raise TypeError("Expected argument 'service_types' to be a list")
        pulumi.set(__self__, "service_types", service_types)
        if subnet_id and not isinstance(subnet_id, str):
            raise TypeError("Expected argument 'subnet_id' to be a str")
        pulumi.set(__self__, "subnet_id", subnet_id)
        if subnetpool_id and not isinstance(subnetpool_id, str):
            raise TypeError("Expected argument 'subnetpool_id' to be a str")
        pulumi.set(__self__, "subnetpool_id", subnetpool_id)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)
        if tenant_id and not isinstance(tenant_id, str):
            raise TypeError("Expected argument 'tenant_id' to be a str")
        pulumi.set(__self__, "tenant_id", tenant_id)

    @_builtins.property
    @pulumi.getter(name="allTags")
    def all_tags(self) -> Sequence[_builtins.str]:
        """
        A set of string tags applied on the subnet.
        """
        return pulumi.get(self, "all_tags")

    @_builtins.property
    @pulumi.getter(name="allocationPools")
    def allocation_pools(self) -> Sequence['outputs.GetSubnetAllocationPoolResult']:
        """
        Allocation pools of the subnet.
        """
        return pulumi.get(self, "allocation_pools")

    @_builtins.property
    @pulumi.getter
    def cidr(self) -> _builtins.str:
        return pulumi.get(self, "cidr")

    @_builtins.property
    @pulumi.getter
    def description(self) -> _builtins.str:
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter(name="dhcpEnabled")
    def dhcp_enabled(self) -> Optional[_builtins.bool]:
        return pulumi.get(self, "dhcp_enabled")

    @_builtins.property
    @pulumi.getter(name="dnsNameservers")
    def dns_nameservers(self) -> Sequence[_builtins.str]:
        """
        DNS Nameservers of the subnet.
        """
        return pulumi.get(self, "dns_nameservers")

    @_builtins.property
    @pulumi.getter(name="dnsPublishFixedIp")
    def dns_publish_fixed_ip(self) -> Optional[_builtins.bool]:
        return pulumi.get(self, "dns_publish_fixed_ip")

    @_builtins.property
    @pulumi.getter(name="enableDhcp")
    def enable_dhcp(self) -> _builtins.bool:
        """
        Whether the subnet has DHCP enabled or not.
        """
        return pulumi.get(self, "enable_dhcp")

    @_builtins.property
    @pulumi.getter(name="gatewayIp")
    def gateway_ip(self) -> _builtins.str:
        return pulumi.get(self, "gateway_ip")

    @_builtins.property
    @pulumi.getter(name="hostRoutes")
    def host_routes(self) -> Sequence['outputs.GetSubnetHostRouteResult']:
        """
        Host Routes of the subnet.
        """
        return pulumi.get(self, "host_routes")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="ipVersion")
    def ip_version(self) -> _builtins.int:
        return pulumi.get(self, "ip_version")

    @_builtins.property
    @pulumi.getter(name="ipv6AddressMode")
    def ipv6_address_mode(self) -> _builtins.str:
        return pulumi.get(self, "ipv6_address_mode")

    @_builtins.property
    @pulumi.getter(name="ipv6RaMode")
    def ipv6_ra_mode(self) -> _builtins.str:
        return pulumi.get(self, "ipv6_ra_mode")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="networkId")
    def network_id(self) -> _builtins.str:
        return pulumi.get(self, "network_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        """
        See Argument Reference above.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="segmentId")
    def segment_id(self) -> _builtins.str:
        return pulumi.get(self, "segment_id")

    @_builtins.property
    @pulumi.getter(name="serviceTypes")
    def service_types(self) -> Sequence[_builtins.str]:
        """
        Service types of the subnet.
        """
        return pulumi.get(self, "service_types")

    @_builtins.property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> _builtins.str:
        return pulumi.get(self, "subnet_id")

    @_builtins.property
    @pulumi.getter(name="subnetpoolId")
    def subnetpool_id(self) -> _builtins.str:
        return pulumi.get(self, "subnetpool_id")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[Sequence[_builtins.str]]:
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter(name="tenantId")
    def tenant_id(self) -> _builtins.str:
        return pulumi.get(self, "tenant_id")


class AwaitableGetSubnetResult(GetSubnetResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetSubnetResult(
            all_tags=self.all_tags,
            allocation_pools=self.allocation_pools,
            cidr=self.cidr,
            description=self.description,
            dhcp_enabled=self.dhcp_enabled,
            dns_nameservers=self.dns_nameservers,
            dns_publish_fixed_ip=self.dns_publish_fixed_ip,
            enable_dhcp=self.enable_dhcp,
            gateway_ip=self.gateway_ip,
            host_routes=self.host_routes,
            id=self.id,
            ip_version=self.ip_version,
            ipv6_address_mode=self.ipv6_address_mode,
            ipv6_ra_mode=self.ipv6_ra_mode,
            name=self.name,
            network_id=self.network_id,
            region=self.region,
            segment_id=self.segment_id,
            service_types=self.service_types,
            subnet_id=self.subnet_id,
            subnetpool_id=self.subnetpool_id,
            tags=self.tags,
            tenant_id=self.tenant_id)


def get_subnet(cidr: Optional[_builtins.str] = None,
               description: Optional[_builtins.str] = None,
               dhcp_enabled: Optional[_builtins.bool] = None,
               dns_publish_fixed_ip: Optional[_builtins.bool] = None,
               gateway_ip: Optional[_builtins.str] = None,
               ip_version: Optional[_builtins.int] = None,
               ipv6_address_mode: Optional[_builtins.str] = None,
               ipv6_ra_mode: Optional[_builtins.str] = None,
               name: Optional[_builtins.str] = None,
               network_id: Optional[_builtins.str] = None,
               region: Optional[_builtins.str] = None,
               segment_id: Optional[_builtins.str] = None,
               subnet_id: Optional[_builtins.str] = None,
               subnetpool_id: Optional[_builtins.str] = None,
               tags: Optional[Sequence[_builtins.str]] = None,
               tenant_id: Optional[_builtins.str] = None,
               opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetSubnetResult:
    """
    Use this data source to get the ID of an available OpenStack subnet.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    subnet1 = openstack.networking.get_subnet(name="subnet_1")
    ```


    :param _builtins.str cidr: The CIDR of the subnet.
    :param _builtins.str description: Human-readable description of the subnet.
    :param _builtins.bool dhcp_enabled: If the subnet has DHCP enabled.
    :param _builtins.bool dns_publish_fixed_ip: If the subnet publishes DNS records.
    :param _builtins.str gateway_ip: The IP of the subnet's gateway.
    :param _builtins.int ip_version: The IP version of the subnet (either 4 or 6).
    :param _builtins.str ipv6_address_mode: The IPv6 address mode. Valid values are
           `dhcpv6-stateful`, `dhcpv6-stateless`, or `slaac`.
    :param _builtins.str ipv6_ra_mode: The IPv6 Router Advertisement mode. Valid values
           are `dhcpv6-stateful`, `dhcpv6-stateless`, or `slaac`.
    :param _builtins.str name: The name of the subnet.
    :param _builtins.str network_id: The ID of the network the subnet belongs to.
    :param _builtins.str region: The region in which to obtain the V2 Neutron client.
           A Neutron client is needed to retrieve subnet ids. If omitted, the
           `region` argument of the provider is used.
    :param _builtins.str segment_id: The ID of the segment the subnet belongs to.
           Available when neutron segment extension is enabled.
    :param _builtins.str subnet_id: The ID of the subnet.
    :param _builtins.str subnetpool_id: The ID of the subnetpool associated with the subnet.
    :param Sequence[_builtins.str] tags: The list of subnet tags to filter.
    :param _builtins.str tenant_id: The owner of the subnet.
    """
    __args__ = dict()
    __args__['cidr'] = cidr
    __args__['description'] = description
    __args__['dhcpEnabled'] = dhcp_enabled
    __args__['dnsPublishFixedIp'] = dns_publish_fixed_ip
    __args__['gatewayIp'] = gateway_ip
    __args__['ipVersion'] = ip_version
    __args__['ipv6AddressMode'] = ipv6_address_mode
    __args__['ipv6RaMode'] = ipv6_ra_mode
    __args__['name'] = name
    __args__['networkId'] = network_id
    __args__['region'] = region
    __args__['segmentId'] = segment_id
    __args__['subnetId'] = subnet_id
    __args__['subnetpoolId'] = subnetpool_id
    __args__['tags'] = tags
    __args__['tenantId'] = tenant_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('openstack:networking/getSubnet:getSubnet', __args__, opts=opts, typ=GetSubnetResult).value

    return AwaitableGetSubnetResult(
        all_tags=pulumi.get(__ret__, 'all_tags'),
        allocation_pools=pulumi.get(__ret__, 'allocation_pools'),
        cidr=pulumi.get(__ret__, 'cidr'),
        description=pulumi.get(__ret__, 'description'),
        dhcp_enabled=pulumi.get(__ret__, 'dhcp_enabled'),
        dns_nameservers=pulumi.get(__ret__, 'dns_nameservers'),
        dns_publish_fixed_ip=pulumi.get(__ret__, 'dns_publish_fixed_ip'),
        enable_dhcp=pulumi.get(__ret__, 'enable_dhcp'),
        gateway_ip=pulumi.get(__ret__, 'gateway_ip'),
        host_routes=pulumi.get(__ret__, 'host_routes'),
        id=pulumi.get(__ret__, 'id'),
        ip_version=pulumi.get(__ret__, 'ip_version'),
        ipv6_address_mode=pulumi.get(__ret__, 'ipv6_address_mode'),
        ipv6_ra_mode=pulumi.get(__ret__, 'ipv6_ra_mode'),
        name=pulumi.get(__ret__, 'name'),
        network_id=pulumi.get(__ret__, 'network_id'),
        region=pulumi.get(__ret__, 'region'),
        segment_id=pulumi.get(__ret__, 'segment_id'),
        service_types=pulumi.get(__ret__, 'service_types'),
        subnet_id=pulumi.get(__ret__, 'subnet_id'),
        subnetpool_id=pulumi.get(__ret__, 'subnetpool_id'),
        tags=pulumi.get(__ret__, 'tags'),
        tenant_id=pulumi.get(__ret__, 'tenant_id'))
def get_subnet_output(cidr: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      description: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      dhcp_enabled: Optional[pulumi.Input[Optional[_builtins.bool]]] = None,
                      dns_publish_fixed_ip: Optional[pulumi.Input[Optional[_builtins.bool]]] = None,
                      gateway_ip: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      ip_version: Optional[pulumi.Input[Optional[_builtins.int]]] = None,
                      ipv6_address_mode: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      ipv6_ra_mode: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      network_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      region: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      segment_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      subnet_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      subnetpool_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      tags: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                      tenant_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                      opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetSubnetResult]:
    """
    Use this data source to get the ID of an available OpenStack subnet.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    subnet1 = openstack.networking.get_subnet(name="subnet_1")
    ```


    :param _builtins.str cidr: The CIDR of the subnet.
    :param _builtins.str description: Human-readable description of the subnet.
    :param _builtins.bool dhcp_enabled: If the subnet has DHCP enabled.
    :param _builtins.bool dns_publish_fixed_ip: If the subnet publishes DNS records.
    :param _builtins.str gateway_ip: The IP of the subnet's gateway.
    :param _builtins.int ip_version: The IP version of the subnet (either 4 or 6).
    :param _builtins.str ipv6_address_mode: The IPv6 address mode. Valid values are
           `dhcpv6-stateful`, `dhcpv6-stateless`, or `slaac`.
    :param _builtins.str ipv6_ra_mode: The IPv6 Router Advertisement mode. Valid values
           are `dhcpv6-stateful`, `dhcpv6-stateless`, or `slaac`.
    :param _builtins.str name: The name of the subnet.
    :param _builtins.str network_id: The ID of the network the subnet belongs to.
    :param _builtins.str region: The region in which to obtain the V2 Neutron client.
           A Neutron client is needed to retrieve subnet ids. If omitted, the
           `region` argument of the provider is used.
    :param _builtins.str segment_id: The ID of the segment the subnet belongs to.
           Available when neutron segment extension is enabled.
    :param _builtins.str subnet_id: The ID of the subnet.
    :param _builtins.str subnetpool_id: The ID of the subnetpool associated with the subnet.
    :param Sequence[_builtins.str] tags: The list of subnet tags to filter.
    :param _builtins.str tenant_id: The owner of the subnet.
    """
    __args__ = dict()
    __args__['cidr'] = cidr
    __args__['description'] = description
    __args__['dhcpEnabled'] = dhcp_enabled
    __args__['dnsPublishFixedIp'] = dns_publish_fixed_ip
    __args__['gatewayIp'] = gateway_ip
    __args__['ipVersion'] = ip_version
    __args__['ipv6AddressMode'] = ipv6_address_mode
    __args__['ipv6RaMode'] = ipv6_ra_mode
    __args__['name'] = name
    __args__['networkId'] = network_id
    __args__['region'] = region
    __args__['segmentId'] = segment_id
    __args__['subnetId'] = subnet_id
    __args__['subnetpoolId'] = subnetpool_id
    __args__['tags'] = tags
    __args__['tenantId'] = tenant_id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('openstack:networking/getSubnet:getSubnet', __args__, opts=opts, typ=GetSubnetResult)
    return __ret__.apply(lambda __response__: GetSubnetResult(
        all_tags=pulumi.get(__response__, 'all_tags'),
        allocation_pools=pulumi.get(__response__, 'allocation_pools'),
        cidr=pulumi.get(__response__, 'cidr'),
        description=pulumi.get(__response__, 'description'),
        dhcp_enabled=pulumi.get(__response__, 'dhcp_enabled'),
        dns_nameservers=pulumi.get(__response__, 'dns_nameservers'),
        dns_publish_fixed_ip=pulumi.get(__response__, 'dns_publish_fixed_ip'),
        enable_dhcp=pulumi.get(__response__, 'enable_dhcp'),
        gateway_ip=pulumi.get(__response__, 'gateway_ip'),
        host_routes=pulumi.get(__response__, 'host_routes'),
        id=pulumi.get(__response__, 'id'),
        ip_version=pulumi.get(__response__, 'ip_version'),
        ipv6_address_mode=pulumi.get(__response__, 'ipv6_address_mode'),
        ipv6_ra_mode=pulumi.get(__response__, 'ipv6_ra_mode'),
        name=pulumi.get(__response__, 'name'),
        network_id=pulumi.get(__response__, 'network_id'),
        region=pulumi.get(__response__, 'region'),
        segment_id=pulumi.get(__response__, 'segment_id'),
        service_types=pulumi.get(__response__, 'service_types'),
        subnet_id=pulumi.get(__response__, 'subnet_id'),
        subnetpool_id=pulumi.get(__response__, 'subnetpool_id'),
        tags=pulumi.get(__response__, 'tags'),
        tenant_id=pulumi.get(__response__, 'tenant_id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['EndpointV3Args', 'EndpointV3']

@pulumi.input_type
class EndpointV3Args:
    def __init__(__self__, *,
                 endpoint_region: pulumi.Input[_builtins.str],
                 service_id: pulumi.Input[_builtins.str],
                 url: pulumi.Input[_builtins.str],
                 interface: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a EndpointV3 resource.

        :param pulumi.Input[_builtins.str] endpoint_region: The endpoint region. The `region` and
               `endpoint_region` can be different.
        :param pulumi.Input[_builtins.str] service_id: The endpoint service ID.
        :param pulumi.Input[_builtins.str] url: The endpoint url.
        :param pulumi.Input[_builtins.str] interface: The endpoint interface. Valid values are `public`,
               `internal` and `admin`. Default value is `public`
        :param pulumi.Input[_builtins.str] name: The endpoint name.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Keystone client.
               If omitted, the `region` argument of the provider is used.
        """
        pulumi.set(__self__, "endpoint_region", endpoint_region)
        pulumi.set(__self__, "service_id", service_id)
        pulumi.set(__self__, "url", url)
        if interface is not None:
            pulumi.set(__self__, "interface", interface)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter(name="endpointRegion")
    def endpoint_region(self) -> pulumi.Input[_builtins.str]:
        """
        The endpoint region. The `region` and
        `endpoint_region` can be different.
        """
        return pulumi.get(self, "endpoint_region")

    @endpoint_region.setter
    def endpoint_region(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "endpoint_region", value)

    @_builtins.property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> pulumi.Input[_builtins.str]:
        """
        The endpoint service ID.
        """
        return pulumi.get(self, "service_id")

    @service_id.setter
    def service_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "service_id", value)

    @_builtins.property
    @pulumi.getter
    def url(self) -> pulumi.Input[_builtins.str]:
        """
        The endpoint url.
        """
        return pulumi.get(self, "url")

    @url.setter
    def url(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "url", value)

    @_builtins.property
    @pulumi.getter
    def interface(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The endpoint interface. Valid values are `public`,
        `internal` and `admin`. Default value is `public`
        """
        return pulumi.get(self, "interface")

    @interface.setter
    def interface(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "interface", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The endpoint name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V3 Keystone client.
        If omitted, the `region` argument of the provider is used.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class _EndpointV3State:
    def __init__(__self__, *,
                 endpoint_region: Optional[pulumi.Input[_builtins.str]] = None,
                 interface: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 service_id: Optional[pulumi.Input[_builtins.str]] = None,
                 service_name: Optional[pulumi.Input[_builtins.str]] = None,
                 service_type: Optional[pulumi.Input[_builtins.str]] = None,
                 url: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering EndpointV3 resources.

        :param pulumi.Input[_builtins.str] endpoint_region: The endpoint region. The `region` and
               `endpoint_region` can be different.
        :param pulumi.Input[_builtins.str] interface: The endpoint interface. Valid values are `public`,
               `internal` and `admin`. Default value is `public`
        :param pulumi.Input[_builtins.str] name: The endpoint name.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Keystone client.
               If omitted, the `region` argument of the provider is used.
        :param pulumi.Input[_builtins.str] service_id: The endpoint service ID.
        :param pulumi.Input[_builtins.str] service_name: The service name of the endpoint.
        :param pulumi.Input[_builtins.str] service_type: The service type of the endpoint.
        :param pulumi.Input[_builtins.str] url: The endpoint url.
        """
        if endpoint_region is not None:
            pulumi.set(__self__, "endpoint_region", endpoint_region)
        if interface is not None:
            pulumi.set(__self__, "interface", interface)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if service_id is not None:
            pulumi.set(__self__, "service_id", service_id)
        if service_name is not None:
            pulumi.set(__self__, "service_name", service_name)
        if service_type is not None:
            pulumi.set(__self__, "service_type", service_type)
        if url is not None:
            pulumi.set(__self__, "url", url)

    @_builtins.property
    @pulumi.getter(name="endpointRegion")
    def endpoint_region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The endpoint region. The `region` and
        `endpoint_region` can be different.
        """
        return pulumi.get(self, "endpoint_region")

    @endpoint_region.setter
    def endpoint_region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "endpoint_region", value)

    @_builtins.property
    @pulumi.getter
    def interface(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The endpoint interface. Valid values are `public`,
        `internal` and `admin`. Default value is `public`
        """
        return pulumi.get(self, "interface")

    @interface.setter
    def interface(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "interface", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The endpoint name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V3 Keystone client.
        If omitted, the `region` argument of the provider is used.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The endpoint service ID.
        """
        return pulumi.get(self, "service_id")

    @service_id.setter
    def service_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_id", value)

    @_builtins.property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The service name of the endpoint.
        """
        return pulumi.get(self, "service_name")

    @service_name.setter
    def service_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_name", value)

    @_builtins.property
    @pulumi.getter(name="serviceType")
    def service_type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The service type of the endpoint.
        """
        return pulumi.get(self, "service_type")

    @service_type.setter
    def service_type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_type", value)

    @_builtins.property
    @pulumi.getter
    def url(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The endpoint url.
        """
        return pulumi.get(self, "url")

    @url.setter
    def url(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "url", value)


@pulumi.type_token("openstack:identity/endpointV3:EndpointV3")
class EndpointV3(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 endpoint_region: Optional[pulumi.Input[_builtins.str]] = None,
                 interface: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 service_id: Optional[pulumi.Input[_builtins.str]] = None,
                 url: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Manages a V3 Endpoint resource within OpenStack Keystone.

        > **Note:** This usually requires admin privileges.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_openstack as openstack

        service1 = openstack.identity.ServiceV3("service_1",
            name="my-service",
            type="my-service-type")
        endpoint1 = openstack.identity.EndpointV3("endpoint_1",
            name="my-endpoint",
            service_id=service1.id,
            endpoint_region=service1.region,
            url="http://my-endpoint")
        ```

        ## Import

        Endpoints can be imported using the `id`, e.g.

        ```sh
        $ pulumi import openstack:identity/endpointV3:EndpointV3 endpoint_1 5392472b-106a-4845-90c6-7c8445f18770
        ```


        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] endpoint_region: The endpoint region. The `region` and
               `endpoint_region` can be different.
        :param pulumi.Input[_builtins.str] interface: The endpoint interface. Valid values are `public`,
               `internal` and `admin`. Default value is `public`
        :param pulumi.Input[_builtins.str] name: The endpoint name.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Keystone client.
               If omitted, the `region` argument of the provider is used.
        :param pulumi.Input[_builtins.str] service_id: The endpoint service ID.
        :param pulumi.Input[_builtins.str] url: The endpoint url.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: EndpointV3Args,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a V3 Endpoint resource within OpenStack Keystone.

        > **Note:** This usually requires admin privileges.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_openstack as openstack

        service1 = openstack.identity.ServiceV3("service_1",
            name="my-service",
            type="my-service-type")
        endpoint1 = openstack.identity.EndpointV3("endpoint_1",
            name="my-endpoint",
            service_id=service1.id,
            endpoint_region=service1.region,
            url="http://my-endpoint")
        ```

        ## Import

        Endpoints can be imported using the `id`, e.g.

        ```sh
        $ pulumi import openstack:identity/endpointV3:EndpointV3 endpoint_1 5392472b-106a-4845-90c6-7c8445f18770
        ```


        :param str resource_name: The name of the resource.
        :param EndpointV3Args args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(EndpointV3Args, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 endpoint_region: Optional[pulumi.Input[_builtins.str]] = None,
                 interface: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 service_id: Optional[pulumi.Input[_builtins.str]] = None,
                 url: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = EndpointV3Args.__new__(EndpointV3Args)

            if endpoint_region is None and not opts.urn:
                raise TypeError("Missing required property 'endpoint_region'")
            __props__.__dict__["endpoint_region"] = endpoint_region
            __props__.__dict__["interface"] = interface
            __props__.__dict__["name"] = name
            __props__.__dict__["region"] = region
            if service_id is None and not opts.urn:
                raise TypeError("Missing required property 'service_id'")
            __props__.__dict__["service_id"] = service_id
            if url is None and not opts.urn:
                raise TypeError("Missing required property 'url'")
            __props__.__dict__["url"] = url
            __props__.__dict__["service_name"] = None
            __props__.__dict__["service_type"] = None
        super(EndpointV3, __self__).__init__(
            'openstack:identity/endpointV3:EndpointV3',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            endpoint_region: Optional[pulumi.Input[_builtins.str]] = None,
            interface: Optional[pulumi.Input[_builtins.str]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            service_id: Optional[pulumi.Input[_builtins.str]] = None,
            service_name: Optional[pulumi.Input[_builtins.str]] = None,
            service_type: Optional[pulumi.Input[_builtins.str]] = None,
            url: Optional[pulumi.Input[_builtins.str]] = None) -> 'EndpointV3':
        """
        Get an existing EndpointV3 resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] endpoint_region: The endpoint region. The `region` and
               `endpoint_region` can be different.
        :param pulumi.Input[_builtins.str] interface: The endpoint interface. Valid values are `public`,
               `internal` and `admin`. Default value is `public`
        :param pulumi.Input[_builtins.str] name: The endpoint name.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Keystone client.
               If omitted, the `region` argument of the provider is used.
        :param pulumi.Input[_builtins.str] service_id: The endpoint service ID.
        :param pulumi.Input[_builtins.str] service_name: The service name of the endpoint.
        :param pulumi.Input[_builtins.str] service_type: The service type of the endpoint.
        :param pulumi.Input[_builtins.str] url: The endpoint url.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _EndpointV3State.__new__(_EndpointV3State)

        __props__.__dict__["endpoint_region"] = endpoint_region
        __props__.__dict__["interface"] = interface
        __props__.__dict__["name"] = name
        __props__.__dict__["region"] = region
        __props__.__dict__["service_id"] = service_id
        __props__.__dict__["service_name"] = service_name
        __props__.__dict__["service_type"] = service_type
        __props__.__dict__["url"] = url
        return EndpointV3(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="endpointRegion")
    def endpoint_region(self) -> pulumi.Output[_builtins.str]:
        """
        The endpoint region. The `region` and
        `endpoint_region` can be different.
        """
        return pulumi.get(self, "endpoint_region")

    @_builtins.property
    @pulumi.getter
    def interface(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The endpoint interface. Valid values are `public`,
        `internal` and `admin`. Default value is `public`
        """
        return pulumi.get(self, "interface")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The endpoint name.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region in which to obtain the V3 Keystone client.
        If omitted, the `region` argument of the provider is used.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> pulumi.Output[_builtins.str]:
        """
        The endpoint service ID.
        """
        return pulumi.get(self, "service_id")

    @_builtins.property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> pulumi.Output[_builtins.str]:
        """
        The service name of the endpoint.
        """
        return pulumi.get(self, "service_name")

    @_builtins.property
    @pulumi.getter(name="serviceType")
    def service_type(self) -> pulumi.Output[_builtins.str]:
        """
        The service type of the endpoint.
        """
        return pulumi.get(self, "service_type")

    @_builtins.property
    @pulumi.getter
    def url(self) -> pulumi.Output[_builtins.str]:
        """
        The endpoint url.
        """
        return pulumi.get(self, "url")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'GetFlavorV2Result',
    'AwaitableGetFlavorV2Result',
    'get_flavor_v2',
    'get_flavor_v2_output',
]

@pulumi.output_type
class GetFlavorV2Result:
    """
    A collection of values returned by getFlavorV2.
    """
    def __init__(__self__, description=None, enabled=None, flavor_id=None, flavor_profile_id=None, id=None, name=None, region=None):
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if enabled and not isinstance(enabled, bool):
            raise TypeError("Expected argument 'enabled' to be a bool")
        pulumi.set(__self__, "enabled", enabled)
        if flavor_id and not isinstance(flavor_id, str):
            raise TypeError("Expected argument 'flavor_id' to be a str")
        pulumi.set(__self__, "flavor_id", flavor_id)
        if flavor_profile_id and not isinstance(flavor_profile_id, str):
            raise TypeError("Expected argument 'flavor_profile_id' to be a str")
        pulumi.set(__self__, "flavor_profile_id", flavor_profile_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter
    def description(self) -> _builtins.str:
        """
        The description of the flavor.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter
    def enabled(self) -> _builtins.bool:
        """
        Is the flavor enabled.
        """
        return pulumi.get(self, "enabled")

    @_builtins.property
    @pulumi.getter(name="flavorId")
    def flavor_id(self) -> _builtins.str:
        """
        The ID of the flavor.
        """
        return pulumi.get(self, "flavor_id")

    @_builtins.property
    @pulumi.getter(name="flavorProfileId")
    def flavor_profile_id(self) -> _builtins.str:
        """
        The ID of the flavor profile.
        """
        return pulumi.get(self, "flavor_profile_id")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        The name of the flavor.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        return pulumi.get(self, "region")


class AwaitableGetFlavorV2Result(GetFlavorV2Result):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetFlavorV2Result(
            description=self.description,
            enabled=self.enabled,
            flavor_id=self.flavor_id,
            flavor_profile_id=self.flavor_profile_id,
            id=self.id,
            name=self.name,
            region=self.region)


def get_flavor_v2(flavor_id: Optional[_builtins.str] = None,
                  name: Optional[_builtins.str] = None,
                  region: Optional[_builtins.str] = None,
                  opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetFlavorV2Result:
    """
    Use this data source to get the ID of an OpenStack Load Balancer flavor.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    flavor1 = openstack.loadbalancer.get_flavor_v2(name="flavor_1")
    ```


    :param _builtins.str flavor_id: The ID of the flavor. Exactly one of `name`, `flavor_id` is required to be set.
    :param _builtins.str name: The name of the flavor. Exactly one of `name`, `flavor_id` is required to be set.
    :param _builtins.str region: The region in which to obtain the V2 Load Balancer client.
           If omitted, the `region` argument of the provider is used.
    """
    __args__ = dict()
    __args__['flavorId'] = flavor_id
    __args__['name'] = name
    __args__['region'] = region
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('openstack:loadbalancer/getFlavorV2:getFlavorV2', __args__, opts=opts, typ=GetFlavorV2Result).value

    return AwaitableGetFlavorV2Result(
        description=pulumi.get(__ret__, 'description'),
        enabled=pulumi.get(__ret__, 'enabled'),
        flavor_id=pulumi.get(__ret__, 'flavor_id'),
        flavor_profile_id=pulumi.get(__ret__, 'flavor_profile_id'),
        id=pulumi.get(__ret__, 'id'),
        name=pulumi.get(__ret__, 'name'),
        region=pulumi.get(__ret__, 'region'))
def get_flavor_v2_output(flavor_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                         name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                         region: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                         opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetFlavorV2Result]:
    """
    Use this data source to get the ID of an OpenStack Load Balancer flavor.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    flavor1 = openstack.loadbalancer.get_flavor_v2(name="flavor_1")
    ```


    :param _builtins.str flavor_id: The ID of the flavor. Exactly one of `name`, `flavor_id` is required to be set.
    :param _builtins.str name: The name of the flavor. Exactly one of `name`, `flavor_id` is required to be set.
    :param _builtins.str region: The region in which to obtain the V2 Load Balancer client.
           If omitted, the `region` argument of the provider is used.
    """
    __args__ = dict()
    __args__['flavorId'] = flavor_id
    __args__['name'] = name
    __args__['region'] = region
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('openstack:loadbalancer/getFlavorV2:getFlavorV2', __args__, opts=opts, typ=GetFlavorV2Result)
    return __ret__.apply(lambda __response__: GetFlavorV2Result(
        description=pulumi.get(__response__, 'description'),
        enabled=pulumi.get(__response__, 'enabled'),
        flavor_id=pulumi.get(__response__, 'flavor_id'),
        flavor_profile_id=pulumi.get(__response__, 'flavor_profile_id'),
        id=pulumi.get(__response__, 'id'),
        name=pulumi.get(__response__, 'name'),
        region=pulumi.get(__response__, 'region')))
